--Top Level

library IEEE;
use IEEE.std_logic_1164.all;

entity top is
    port (
        clk: in STD_LOGIC;
        P16: in STD_LOGIC;
        din: in STD_LOGIC_VECTOR (5 downto 0);
        dout: out STD_LOGIC_VECTOR (3 downto 0);
        a: out STD_LOGIC;
        b: out STD_LOGIC;
        c: out STD_LOGIC;
        d: out STD_LOGIC;
        e: out STD_LOGIC;
        f: out STD_LOGIC;
        g: out STD_LOGIC
    );
end top;

architecture top_arch of top is
component sppinterf 
    generic (ADDR_SIZE: integer := 2);
    port (
        clk: in STD_LOGIC;
        P16: in STD_LOGIC;
        reset: out STD_LOGIC;
	P44: in STD_LOGIC;
	P45: in STD_LOGIC;
	P46: in STD_LOGIC;
	P47: in STD_LOGIC;
	P48: in STD_LOGIC;
	P49: in STD_LOGIC;
	P70: out STD_LOGIC;
	P77: out STD_LOGIC;
	P66: out STD_LOGIC;
	P69: out STD_LOGIC;
        din: in STD_LOGIC_VECTOR(7 downto 0);
	dout: out STD_LOGIC_VECTOR(7 downto 0);
	addr: out STD_LOGIC_VECTOR(ADDR_SIZE-1 downto 0);
	read: out STD_LOGIC;
	write: out STD_LOGIC
    );
end component;
component RegSinc is
    generic(N: integer :=8);
    port (
        clk: in STD_LOGIC;
        reset: in STD_LOGIC;
        load: in STD_LOGIC; 
        din: in STD_LOGIC_VECTOR (N-1 downto 0);
        dout: out STD_LOGIC_VECTOR (N-1 downto 0)
    );
end component;

component segment7kk is
    port (
        entrada: in STD_LOGIC_VECTOR (3 downto 0);
        A: out STD_LOGIC;
        B: out STD_LOGIC;
        C: out STD_LOGIC;
        D: out STD_LOGIC;
        E: out STD_LOGIC;
        F: out STD_LOGIC;
        G: out STD_LOGIC
    );
end component;

component decod is
    generic(
    	N: INTEGER :=2;
    	M: INTEGER :=4
    	);
    	
    port (
    	enable : in STD_LOGIC;
        address: in STD_LOGIC_VECTOR (N-1 downto 0);
        pointers: out STD_LOGIC_VECTOR (M-1 downto 0)
    );
end component;
component Mux4 is
    generic( N:Integer:=8);
    port (
        r0: in STD_LOGIC_VECTOR(N-1 downto 0);
        r1: in STD_LOGIC_VECTOR(N-1 downto 0);
        r2: in STD_LOGIC_VECTOR(N-1 downto 0);
        r3: in STD_LOGIC_VECTOR(N-1 downto 0);
         paddr: in STD_LOGIC_VECTOR(3 downto 0);
        rd: out STD_LOGIC_VECTOR(N-1 downto 0)
    );
end component;
signal wrint,rdint : std_logic;
signal ddisp,data,dw1,dw0,dw2,dw3: STD_LOGIC_VECTOR (7 downto 0);
signal pw,pr : STD_LOGIC_VECTOR (3 downto 0);
signal ad : STD_LOGIC_VECTOR (4 downto 0);
signal rst: std_logic; 

begin

SPP: sppinterf
  generic map(ADDR_SIZE => 4)
  port map(  clk => clk,
        P16 => P16,
        reset => rst,
  	P44 => din(0),
	P45 => din(1),
	P46 => din(2),
	P47 => din(3),
	P48 => din(4),
	P49 => din(5),
	P70 => dout(0),
	P77 => dout(1),
	P66 => dout(2),
	P69 => dout(3),
        din => ddisp,
	dout => data,
	addr => ad(3 downto 0),
	read => rdint,
	write => wrint);


--Four Registers for RAM emulation
reg0: RegSinc 
    generic map(8)
    port map(clk,rst,pw(0),data,dw0);
reg1: RegSinc 
    generic map(8)
    port map(clk,rst,pw(1),data,dw1);
reg2: RegSinc 
    generic map(8)
    port map(clk,rst,pw(2),data,dw2);
reg3: RegSinc 
    generic map(8)
    port map(clk,rst,pw(3),data,dw3);

--Decoder for write Address
decaw: decod
    generic map(2,4)
    port map(wrint,ad(1 downto 0),pw(3 downto 0));
    
--Decoder for Read Address
decar: decod
    generic map(2,4)
    port map(rdint,ad(1 downto 0),pr(3 downto 0));

--Multiplexer for Read Selection
selrd: Mux4    
    generic map(8)
    port map(dw0,dw1,DW2,DW3,pr,ddisp);
  

--For debugging purpose I use the display
disp: segment7kk 
    port map(ad(3 downto 0),A,B,C,D,E,F,G);
    
end top_arch;
