--Top Level
--Miguel A. Aguirre Echanove. University of Sevilla (SPAIN)
--Dpt. of Ingenieria Electronica. aguirre@gte.esi.us.es
--
library IEEE;
use IEEE.std_logic_1164.all;

entity sppinterf is
    port (
        clk: in STD_LOGIC;
        rst: in STD_LOGIC; -- For XS40 v1.4
        din: in STD_LOGIC_VECTOR (5 downto 0);
        wrextb: in STD_LOGIC; -- For XS40 v1.4
--        rdextb: in STD_LOGIC; 
        dout: out STD_LOGIC_VECTOR (3 downto 0);
        a: out STD_LOGIC;
        b: out STD_LOGIC;
        c: out STD_LOGIC;
        d: out STD_LOGIC;
        e: out STD_LOGIC;
        f: out STD_LOGIC;
        g: out STD_LOGIC
    );
end sppinterf;

architecture sppinterf_arch of sppinterf is
component TDO port (O: in std_logic); end component;
component TDI port (I: out std_logic); end component;
component TCK port (I: out std_logic); end component;
component TMS port (I: out std_logic); end component;
component MD0 port (I: out std_logic); end component;
component MD1 port (O: in std_logic); end component;
component MD2 port (I: out std_logic); end component;
component IBUF port(I: in std_logic;O: out std_logic); end component;
component OBUF port(I: in std_logic;O: out std_logic); end component;
--component IFD port (C,D: in std_logic; Q: out std_logic); end component;
component RegSinc is
    generic(N: integer :=8);
    port (
        clk: in STD_LOGIC;
        reset: in STD_LOGIC;
        load: in STD_LOGIC; 
        din: in STD_LOGIC_VECTOR (N-1 downto 0);
        dout: out STD_LOGIC_VECTOR (N-1 downto 0)
    );
end component;
component FFcleaner is
    port (
        D: in STD_LOGIC;
        CLK: in STD_LOGIC;
        RST: in STD_LOGIC;
        Q: out STD_LOGIC
    );
end component;
component fsmrdwr is
port (
        clk: in STD_LOGIC;
        rst: in STD_LOGIC;
        wrexb: in STD_LOGIC;
        rdexb: in STD_LOGIC;
        wrint: out STD_LOGIC;
        aleint: out STD_LOGIC;
        rdint: out STD_LOGIC;
        nibble: out STD_LOGIC
    );
end component;
component segment7kk is
    port (
        entrada: in STD_LOGIC_VECTOR (3 downto 0);
        A: out STD_LOGIC;
        B: out STD_LOGIC;
        C: out STD_LOGIC;
        D: out STD_LOGIC;
        E: out STD_LOGIC;
        F: out STD_LOGIC;
        G: out STD_LOGIC
    );
end component;
component Muxsal is
    port (
        I0: in STD_LOGIC;
        I1: in STD_LOGIC;
        I2: in STD_LOGIC;
        I3: in STD_LOGIC;
        I4: in STD_LOGIC;
        I5: in STD_LOGIC;
        I6: in STD_LOGIC;
        I7: in STD_LOGIC;
        Nibble: in STD_LOGIC;
        datard: out STD_LOGIC_VECTOR (4 downto 0)
    );
end component;
component decod is
    generic(
    	N: INTEGER :=2;
    	M: INTEGER :=4
    	);
    	
    port (
    	enable : in STD_LOGIC;
        address: in STD_LOGIC_VECTOR (N-1 downto 0);
        pointers: out STD_LOGIC_VECTOR (M-1 downto 0)
    );
end component;
component Mux4 is
    generic( N:Integer:=8);
    port (
        r0: in STD_LOGIC_VECTOR(N-1 downto 0);
        r1: in STD_LOGIC_VECTOR(N-1 downto 0);
        r2: in STD_LOGIC_VECTOR(N-1 downto 0);
        r3: in STD_LOGIC_VECTOR(N-1 downto 0);
         paddr: in STD_LOGIC_VECTOR(3 downto 0);
        rd: out STD_LOGIC_VECTOR(N-1 downto 0)
    );
end component;
signal writeb,wrint,readb,aleint,rdint,nibble : std_logic;
signal ddisp,data,dw1,dw0,dw2,dw3: STD_LOGIC_VECTOR (7 downto 0);
signal drd,daux : STD_LOGIC_VECTOR (4 downto 0);
signal pw,pr : STD_LOGIC_VECTOR (3 downto 0);
signal ad : STD_LOGIC_VECTOR (7 downto 0);
--signal rstp,rst,wrextbp,wrextb: std_logic; -- For XS40 v1.3 or older
signal rdextb,rdextbp: std_logic;
signal daux4,data6,data7,A1,D1,G1: std_logic;
begin

--I use the special signals with this method
--sio0:  TCK port map (rstp); -- For XS40 v1.3 or older
--bsio0: IBUF port map(rstp,rst); -- For XS40 v1.3 or older 
sio1:  TMS port map (rdextbp); 
bsio1: IBUF port map(rdextbp,rdextb); 
--sio2:  TDI port map (wrextbp);-- For XS40 v1.3 or older
--bsio2: IBUF port map(wrextbp,wrextb); -- For XS40 v1.3 or older
sio3:  TDO port map (daux4);
bsio3: OBUF port map(daux(4),daux4); 
sio4:  MD0 port map (data6);
bsio4: IBUF port map(data6,data(6)); 
sio5:  MD2 port map (data7);
bsio5: IBUF port map(data7,data(7)); 

data(5 downto 0)<=din;


fsm1: fsmrdwr 
    port map(clk,rst,writeb,readb,wrint,aleint,rdint,nibble);
    
--Four Registers for RAM emulation
reg0: RegSinc 
    generic map(8)
    port map(clk,rst,pw(0),data,dw0);
reg1: RegSinc 
    generic map(8)
    port map(clk,rst,pw(1),data,dw1);
reg2: RegSinc 
    generic map(8)
    port map(clk,rst,pw(2),data,dw2);
reg3: RegSinc 
    generic map(8)
    port map(clk,rst,pw(3),data,dw3);

--Decoder for write Address
decaw: decod
    generic map(2,4)
    port map(wrint,ad(1 downto 0),pw(3 downto 0));
    
--Decoder for Read Address
decar: decod
    generic map(2,4)
    port map(rdint,ad(1 downto 0),pr(3 downto 0));

--Multiplexer for Read Selection
selrd: Mux4    
    generic map(8)
    port map(dw0,dw1,DW2,DW3,pr,ddisp);
  
--Address Register
rega: RegSinc 
    generic map(8)
    port map(clk,rst,aleint,data,ad);
    
--Store reading Data in an output register
regr: RegSinc 
    generic map(5)
    port map(clk,rst,rdint,drd,daux);
   dout<=daux(3 downto 0);
   
--For debugging purpose I use the display
disp: segment7kk 
    port map(ad(3 downto 0),A,B,C,D,E,F,G);
    
--This can be included in the input Pad
ffcln1: FFcleaner 
    port map(wrextb,clk,rst,writeb);
ffcln2: FFcleaner 
    port map(rdextb,clk,rst,readb);

--Multiplexer for Nibble selection
muxrd: Muxsal
    port map(ddisp(0),ddisp(1),ddisp(2),ddisp(3),ddisp(4),
    ddisp(5),ddisp(6),ddisp(7),Nibble,drd);
end sppinterf_arch;
